@@ -10,6 +10,8 @@ from cftool.misc import get_counter_from_arr
 
 from ..misc import *
 from ...types import *
+from .binning import BinningBase
+from .binning import BinningError
 
 
 class Recognizer(DataStructure):
@@ -24,13 +26,17 @@ class Recognizer(DataStructure):
         name: str,
         is_np: bool,
         *,
+        binning: str = "auto",
         is_label: bool = False,
         is_valid: Optional[bool] = None,
         task_type: task_type_type = TaskTypes.NONE,
+        labels: Optional[np.ndarray] = None,
         config: Optional[Dict[str, Any]] = None,
     ):
         self.name = name
         self.is_np = is_np
+        self.binning = binning
+        self.labels = labels
         self.is_label = is_label
         self.is_valid = is_valid
         self.task_type = parse_task_type(task_type)
@@ -52,19 +58,8 @@ class Recognizer(DataStructure):
             config = {}
         self.config = config
         self._num_thresh = config.setdefault("numerical_threshold", 0.5)
-        default_bound = 128
+        default_bound = config.setdefault("default_bound", 128)
         self.num_unique_bound = config.setdefault("num_unique_bound", default_bound)
-        self._truncate_ratio = config.setdefault("truncate_ratio", 0.99)
-        if self.num_unique_bound is None:
-            default_fuse_threshold = 1.0 / default_bound
-        else:
-            default_fuse_threshold = 1.0 / self.num_unique_bound
-        self._fuse_thresh = config.setdefault("fuse_threshold", default_fuse_threshold)
-        if self.num_unique_bound is None:
-            default_fuse_fix = int(default_bound // 2)
-        else:
-            default_fuse_fix = int(round(0.5 * self.num_unique_bound))
-        self._num_fuse_fix = config.setdefault("num_fuse_fix", default_fuse_fix)
 
     def _make_invalid_info(
         self,
@@ -103,12 +98,13 @@ class Recognizer(DataStructure):
 
     def _make_dummy_info(
         self,
+        np_flat: np.ndarray,
         unique_values: np.ndarray,
         sorted_counts: np.ndarray,
     ) -> FeatureInfo:
         return FeatureInfo(
             None,
-            None,
+            np_flat,
             num_unique_bound=self.num_unique_bound,
             unique_values_sorted_by_counts=unique_values,
             sorted_counts=sorted_counts,
@@ -116,54 +112,43 @@ class Recognizer(DataStructure):
 
     def _get_transform_dict(
         self,
+        info: FeatureInfo,
         check_nan: bool,
-        values: Union[List[str], List[float]],
+        unique_values: Union[List[str], List[float]],
         sorted_counts: np.ndarray,
-        info: Optional[FeatureInfo] = None,
     ) -> Tuple[transform_dict_type, List[int]]:
-        if info is None:
-            info = self._make_dummy_info(np.array(values), sorted_counts)
-
         def _core(
-            values_: Union[List[str], List[float]],
+            unique_values_: Union[List[str], List[float]],
             indices: Optional[List[int]] = None,
         ) -> transform_dict_type:
             if indices is None:
-                indices = list(range(len(values_)))
-            iterator = zip(indices, values_)
+                indices = list(range(len(unique_values_)))
+            iterator = zip(indices, unique_values_)
             td: transform_dict_type = {}
             if not check_nan:
-                for i_, v_ in iterator:
-                    assert isinstance(v_, (str, float))
-                    td[v_] = i_
+                for i, v in iterator:
+                    assert isinstance(v, (str, float))
+                    td[v] = i
                 return td
-            for i_, v_ in iterator:
-                assert isinstance(v_, float)
-                if math.isnan(v_):
+            for i, v in iterator:
+                assert isinstance(v, float)
+                if math.isnan(v):
                     continue
-                td[v_] = i_
+                td[v] = i
             return td
 
-        if not info.need_truncate:
-            transformed_unique_values = list(range(len(values)))
-            return _core(values), transformed_unique_values
-        # truncate
-        counts_cumsum = np.cumsum(sorted_counts)
-        counts_cumsum_ratio = counts_cumsum / counts_cumsum[-1]
-        truncate_mask = counts_cumsum_ratio >= self._truncate_ratio
-        truncate_idx = np.nonzero(truncate_mask)[0][0]
-        values = values[: truncate_idx + 1]
-        # fuse
-        idx = 0
-        cumulate = 0.0
-        fused_indices = []
-        for i, ratio in enumerate(counts_cumsum_ratio[: truncate_idx + 1]):
-            fused_indices.append(idx)
-            if i < self._num_fuse_fix or ratio >= self._fuse_thresh + cumulate:
-                idx += 1
-                cumulate = ratio
-        transformed_unique_values = sorted(set(fused_indices))
-        return _core(values, fused_indices), transformed_unique_values
+        if self.is_label:
+            return _core(unique_values), list(range(len(unique_values)))
+
+        if self.binning != "auto":
+            binning_type = self.binning
+        else:
+            binning_type = "opt" if self.labels is not None else "fuse"
+        args = binning_type, self.labels, self.task_type, self.config
+        binning = BinningBase.make(*args)
+        results = binning.binning(info, sorted_counts, unique_values)
+        fused_indices, unique_values, transformed_unique_values = results
+        return _core(unique_values, fused_indices), transformed_unique_values
 
     def _check_string_column(
         self,
@@ -204,13 +189,30 @@ class Recognizer(DataStructure):
                 "It'll be excluded since it might be redundant"
             )
             return True, self._make_string_info(None, False, msg)
-        pack = self._get_transform_dict(False, unique_values, sorted_counts)
+        unique_values_arr = np.array(unique_values)
+        dummy_info = self._make_dummy_info(np_flat, unique_values_arr, sorted_counts)
+        args = dummy_info, False, unique_values, sorted_counts
+        try:
+            pack = self._get_transform_dict(*args)
+        except BinningError:
+            self.binning = "fuse"
+            pack = self._get_transform_dict(*args)
+        except Exception as err:
+            msg = (
+                f"column {self.name}, which tends to be string column, "
+                f"cannot be transformed ({err})."
+            )
+            if not self.is_valid:
+                msg = f"{msg} It'll be excluded since it might be redundant"
+                return True, self._make_string_info(None, False, msg)
+            self.binning = "fuse"
+            pack = self._get_transform_dict(*args)
         self.transform_dict, self._transformed_unique_values = pack
         return True, self._make_string_info(
             np_flat,
             True,
             msg,
-            np.array(unique_values),
+            unique_values_arr,
             sorted_counts,
         )
 
@@ -248,7 +250,7 @@ class Recognizer(DataStructure):
         sorted_counts = self.info.sorted_counts
         assert unique_values is not None and sorted_counts is not None
         values = list(map(float, unique_values.tolist()))
-        pack = self._get_transform_dict(True, values, sorted_counts, self.info)
+        pack = self._get_transform_dict(self.info, True, values, sorted_counts)
         transform_dict, self._transformed_unique_values = pack
         if self.info.contains_nan:
             num_transformed_unique = len(self._transformed_unique_values)
@@ -256,15 +258,25 @@ class Recognizer(DataStructure):
             self._transformed_unique_values.append(num_transformed_unique)
         self.transform_dict = transform_dict
 
-    def fit(self, df: dt.Frame) -> "Recognizer":
+    def fit(self, df: dt.Frame, *, is_preset: bool) -> "Recognizer":
+        if self.is_label and is_preset:
+            raise ValueError("`is_preset` should always be False when `is_label`")
         msg: Optional[str]
         dtype = df.stype.dtype
         # check name
-        if len(df.names) != 1 or df.names[0] != self.name:
-            raise ValueError(
-                f"df.name ({df.names[0]}) is not identical with "
-                f"Recognizer.name ({self.name})"
-            )
+        if len(df.names) == 1:
+            if df.names[0] != self.name:
+                raise ValueError(
+                    f"df.name ({df.names[0]}) is not identical with "
+                    f"Recognizer.name ({self.name})"
+                )
+        else:
+            for name in df.names:
+                if not name.startswith(self.name):
+                    raise ValueError(
+                        f"one name ({name}) in df.names does not start with "
+                        f"Recognizer.name ({self.name})"
+                    )
         # check valid
         if self.is_valid is False:
             self.info = FeatureInfo(
@@ -319,7 +331,8 @@ class Recognizer(DataStructure):
             )
             return self._make_invalid_info(msg, contains_nan, nan_mask)
         # check whether it's a numerical column
-        not_int = is_float(dtype) and not np.allclose(np_flat_valid, np_flat_valid_int)
+        like_int = np.allclose(np_flat_valid, np_flat_valid_int)
+        not_int = is_float(dtype) and (is_preset or not like_int)
         is_reg_label = self.is_label and self.task_type.is_reg
         if not_int or is_reg_label:
             msg, is_valid = None, True
@@ -391,7 +404,21 @@ class Recognizer(DataStructure):
             unique_values_sorted_by_counts=unique_values,
             sorted_counts=sorted_counts,
         )
-        self._generate_categorical_transform_dict()
+        try:
+            self._generate_categorical_transform_dict()
+        except BinningError:
+            self.binning = "fuse"
+            self._generate_categorical_transform_dict()
+        except Exception as err:
+            msg = (
+                f"column {self.name}, which tends to be categorical column, "
+                f"cannot be transformed ({err})."
+            )
+            if not self.is_valid:
+                msg = f"{msg} It'll be excluded since it might be redundant"
+                return self._make_invalid_info(msg, contains_nan, nan_mask)
+            self.binning = "fuse"
+            self._generate_categorical_transform_dict()
         return self
 
     def dumps_(self) -> Any:
